@@ -32,6 +32,7 @@
 #include "nemo-file.h"
 #include "nemo-signaller.h"
 #include <eel/eel-stock-dialogs.h>
+#include <eel/eel-string.h>
 
 #include <string.h>
 #include <glib/gi18n-lib.h>
@@ -50,6 +51,10 @@ struct _NemoMimeApplicationChooserDetails {
 	GtkWidget *set_as_default_button;
 	GtkWidget *open_with_widget;
 	GtkWidget *add_button;
+    GtkWidget *custom_picker;
+    GtkWidget *file_button;
+    GAppInfo *custom_info;
+    GtkWidget *custom_entry;
 };
 
 enum {
@@ -59,6 +64,9 @@ enum {
 	NUM_PROPERTIES
 };
 
+#define DEFAULT_APPS "Default Applications"
+#define ADDED_ASS "Added Associations"
+
 static GParamSpec *properties[NUM_PROPERTIES] = { NULL, };
 
 G_DEFINE_TYPE (NemoMimeApplicationChooser, nemo_mime_application_chooser, GTK_TYPE_BOX);
@@ -69,30 +77,22 @@ add_clicked_cb (GtkButton *button,
 {
 	NemoMimeApplicationChooser *chooser = user_data;
 	GAppInfo *info;
-	gchar *message;
-	GError *error = NULL;
 
-	info = gtk_app_chooser_get_app_info (GTK_APP_CHOOSER (chooser->details->open_with_widget));
+    if (!chooser->details->custom_info) {
+        info = gtk_app_chooser_get_app_info (GTK_APP_CHOOSER (chooser->details->open_with_widget));
+        g_app_info_set_as_last_used_for_type (info, chooser->details->content_type, NULL);
+    } else {
+        info = chooser->details->custom_info;
+        g_app_info_set_as_last_used_for_type (info, chooser->details->content_type, NULL);
+    }
 
 	if (info == NULL)
 		return;
 
-	g_app_info_set_as_last_used_for_type (info, chooser->details->content_type, &error);
-
-	if (error != NULL) {
-		message = g_strdup_printf (_("Error while adding \"%s\": %s"),
-					   g_app_info_get_display_name (info), error->message);
-		eel_show_error_dialog (_("Could not add application"),
-				       message,
-				       GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (chooser))));
-		g_error_free (error);
-		g_free (message);
-	} else {		
-		gtk_app_chooser_refresh (GTK_APP_CHOOSER (chooser->details->open_with_widget));
-		g_signal_emit_by_name (nemo_signaller_get_current (), "mime_data_changed");
-	}
-
-	g_object_unref (info);
+    gtk_app_chooser_refresh (GTK_APP_CHOOSER (chooser->details->open_with_widget));
+    gtk_entry_set_text (GTK_ENTRY (chooser->details->custom_entry), "");
+    gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (chooser->details->file_button), "");
+    g_signal_emit_by_name (nemo_signaller_get_current (), "mime_data_changed");
 }
 
 static void
@@ -153,7 +153,8 @@ reset_clicked_cb (GtkButton *button,
 
 	g_app_info_reset_type_associations (chooser->details->content_type);
 	gtk_app_chooser_refresh (GTK_APP_CHOOSER (chooser->details->open_with_widget));
-
+    gtk_entry_set_text (GTK_ENTRY (chooser->details->custom_entry), "");
+    gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (chooser->details->file_button), "");
 	g_signal_emit_by_name (nemo_signaller_get_current (), "mime_data_changed");
 }
 
@@ -163,26 +164,19 @@ set_as_default_clicked_cb (GtkButton *button,
 {
 	NemoMimeApplicationChooser *chooser = user_data;
 	GAppInfo *info;
-	GError *error = NULL;
-	gchar *message = NULL;
 
-	info = gtk_app_chooser_get_app_info (GTK_APP_CHOOSER (chooser->details->open_with_widget));
+    if (!chooser->details->custom_info) {
+        info = gtk_app_chooser_get_app_info (GTK_APP_CHOOSER (chooser->details->open_with_widget));
+        g_app_info_set_as_default_for_type (info, chooser->details->content_type, NULL);
+    } else {
+        info = chooser->details->custom_info;
+        g_app_info_set_as_default_for_type (info, chooser->details->content_type, NULL);
+    }
 
-	g_app_info_set_as_default_for_type (info, chooser->details->content_type,
-					    &error);
-
-	if (error != NULL) {
-		message = g_strdup_printf (_("Error while setting \"%s\" as default application: %s"),
-					   g_app_info_get_display_name (info), error->message);
-		eel_show_error_dialog (_("Could not set as default"),
-				       message,
-				       GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (chooser))));
-	}
-
-	g_object_unref (info);
-
-	gtk_app_chooser_refresh (GTK_APP_CHOOSER (chooser->details->open_with_widget));
-	g_signal_emit_by_name (nemo_signaller_get_current (), "mime_data_changed");
+    gtk_app_chooser_refresh (GTK_APP_CHOOSER (chooser->details->open_with_widget));
+    gtk_entry_set_text (GTK_ENTRY (chooser->details->custom_entry), "");
+    gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (chooser->details->file_button), "");
+    g_signal_emit_by_name (nemo_signaller_get_current (), "mime_data_changed");
 }
 
 static gint
@@ -234,9 +228,139 @@ application_selected_cb (GtkAppChooserWidget *widget,
 	gtk_widget_set_sensitive (chooser->details->add_button,
 				  app_info_can_add (info, chooser->details->content_type));
 
+    gtk_entry_set_text (GTK_ENTRY (chooser->details->custom_entry), "");
+    gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (chooser->details->file_button), "");
+
 	g_object_unref (default_app);
 }
 
+static gboolean
+validate_entry (GtkEntry *entry, const gchar *str)
+{
+    gint argcp = 0;
+    gchar **argvp;
+    gboolean ret = FALSE;
+    if (g_shell_parse_argv (str, &argcp, &argvp, NULL)) {
+        if (argcp > 0) {
+            gchar *path_exec = g_find_program_in_path (argvp[0]);
+            if (path_exec) {
+                ret = TRUE;
+            } else {
+                if (!g_file_test (str, G_FILE_TEST_IS_DIR) &&
+                    g_file_test (str, G_FILE_TEST_IS_EXECUTABLE)) {
+                    ret = TRUE;
+                }
+            }
+            g_free (path_exec);
+        }
+    }
+
+    g_strfreev (argvp);
+
+    if (ret) {
+        gtk_entry_set_icon_from_icon_name (entry,
+                                           GTK_ENTRY_ICON_SECONDARY,
+                                           "ok");
+        gtk_entry_set_icon_tooltip_text (entry,
+                                         GTK_ENTRY_ICON_SECONDARY,
+                                         _("Valid"));
+    } else {
+        gtk_entry_set_icon_from_icon_name (entry,
+                                           GTK_ENTRY_ICON_SECONDARY,
+                                           "stop");
+        gtk_entry_set_icon_tooltip_text (entry,
+                                         GTK_ENTRY_ICON_SECONDARY,
+                                         _("Not a valid executable.  Spaces in the file path must be escaped with backslash (\\)."));
+    }
+
+    return ret;
+}
+
+static gchar *
+get_nice_name (const gchar *entry)
+{
+    gchar *exec_path;
+    gchar *ret = NULL;
+    gint argcp = 0;
+    gchar **argvp;
+
+    if (g_shell_parse_argv (entry, &argcp, &argvp, NULL)) {
+        if (argcp > 0) {
+            exec_path = argvp[0];
+            GFile *file = g_file_new_for_path (exec_path);
+            ret = g_file_get_basename (file);
+            g_object_unref (file);
+        }
+    }
+
+    g_strfreev (argvp);
+    return ret;
+}
+
+
+static void
+custom_entry_changed_cb (GtkEditable *entry, gpointer user_data)
+{
+    NemoMimeApplicationChooser *chooser = user_data;
+
+    const gchar *entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
+    gboolean empty = g_strcmp0 (entry_text, "") == 0;
+
+    if (!empty && validate_entry (GTK_ENTRY (entry), entry_text)) {
+        GAppInfo *default_app;
+        gchar *cl = g_strdup_printf ("%s", entry_text);
+
+        GAppInfo *info = g_app_info_create_from_commandline (cl, get_nice_name (cl),
+                                                             G_APP_INFO_CREATE_NONE, NULL);
+        default_app = g_app_info_get_default_for_type (chooser->details->content_type, FALSE);
+        gtk_widget_set_sensitive (chooser->details->set_as_default_button,
+                      !g_app_info_equal (info, default_app));
+
+        gtk_widget_set_sensitive (chooser->details->add_button,
+                      app_info_can_add (info, chooser->details->content_type));
+
+        g_object_unref (default_app);
+        if (chooser->details->custom_info != NULL) {
+            g_object_unref (chooser->details->custom_info);
+            chooser->details->custom_info = NULL;
+        }
+        chooser->details->custom_info = info;
+
+    } else {
+        if (chooser->details->custom_info != NULL) {
+            g_object_unref (chooser->details->custom_info);
+            chooser->details->custom_info = NULL;
+        }
+
+        if (empty) {
+            gtk_entry_set_icon_from_icon_name (GTK_ENTRY (entry),
+                                               GTK_ENTRY_ICON_SECONDARY,
+                                               NULL);
+            gtk_entry_set_icon_tooltip_text (GTK_ENTRY (entry),
+                                             GTK_ENTRY_ICON_SECONDARY,
+                                             NULL);
+        }
+        
+        gtk_widget_set_sensitive (chooser->details->set_as_default_button, FALSE);
+        gtk_widget_set_sensitive (chooser->details->add_button, FALSE);
+    }
+}
+
+static void
+custom_app_set_cb (GtkFileChooserButton *button,
+                   gpointer user_data)
+{
+    NemoMimeApplicationChooser *chooser = user_data;
+
+    gchar *unescaped = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (button));
+    gchar *escaped = eel_str_escape_spaces (unescaped);
+
+    gtk_entry_set_text (GTK_ENTRY (chooser->details->custom_entry), escaped);
+
+    g_free (unescaped);
+    g_free (escaped);
+}
+
 static char *
 get_extension (const char *basename)
 {
@@ -301,7 +425,7 @@ nemo_mime_application_chooser_apply_labels (NemoMimeApplicationChooser *chooser)
 
 		/* first %s is filename, second %s is mime-type description */
 		emname = g_strdup_printf ("<i>%s</i>", basename);
-		label = g_strdup_printf (_("Select an application to open %s and other files of type \"%s\""),
+		label = g_strdup_printf (_("Select an application in the list to open %s and other files of type \"%s\""),
 					 emname, description);
 
 		g_free (emname);
@@ -316,15 +440,15 @@ nemo_mime_application_chooser_apply_labels (NemoMimeApplicationChooser *chooser)
 	g_free (description);
 }
 
-static void
-show_more_clicked_cb (GtkWidget *button,
-		      gpointer user_data)
+static gboolean
+exec_filter_func (const GtkFileFilterInfo *info, gpointer data)
 {
-	NemoMimeApplicationChooser *chooser = user_data;
+    if (info->contains & GTK_FILE_FILTER_FILENAME) {
+        if (g_file_test (info->filename, G_FILE_TEST_IS_EXECUTABLE))
+            return TRUE;
+    }
 
-	gtk_app_chooser_widget_set_show_other (GTK_APP_CHOOSER_WIDGET (chooser->details->open_with_widget),
-					       TRUE);
-	gtk_widget_hide (button);
+    return FALSE;
 }
 
 static void
@@ -363,28 +487,62 @@ nemo_mime_application_chooser_build_ui (NemoMimeApplicationChooser *chooser)
 			  G_CALLBACK (populate_popup_cb),
 			  chooser);
 
-	button = gtk_button_new_with_label (_("Show other applications"));
-	gtk_box_pack_start (GTK_BOX (chooser->details->open_with_widget), button, FALSE, FALSE, 6);
-	gtk_widget_show_all (button);
+    gtk_app_chooser_widget_set_show_other (GTK_APP_CHOOSER_WIDGET (chooser->details->open_with_widget),
+                          TRUE);
+    gtk_app_chooser_widget_set_show_recommended (GTK_APP_CHOOSER_WIDGET (chooser->details->open_with_widget),
+                          TRUE);
 
-	g_signal_connect (button, "clicked",
-			  G_CALLBACK (show_more_clicked_cb), chooser);
+    GtkWidget *custom_label = gtk_label_new (_("You can also type or select a custom executable file to use to open this file type.  "
+                                               "You can use this command just once, or set it as default for all files of this type."));
+    gtk_misc_set_alignment (GTK_MISC (custom_label), 0.0, 0.5);
+    gtk_label_set_line_wrap (GTK_LABEL (custom_label), TRUE);
+    gtk_label_set_line_wrap_mode (GTK_LABEL (custom_label),
+                                  PANGO_WRAP_WORD_CHAR);
+    gtk_box_pack_start (GTK_BOX (chooser), custom_label, FALSE, FALSE, 0);
+    gtk_widget_show (GTK_WIDGET (custom_label));
+
+    GtkWidget *custom_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+    gtk_box_pack_start (GTK_BOX (chooser), custom_box, TRUE, TRUE, 0);
+
+    GtkWidget *entry = gtk_entry_new ();
+    gtk_box_pack_start (GTK_BOX (custom_box), entry, TRUE, TRUE, 0);
+    gtk_entry_set_placeholder_text (GTK_ENTRY (entry), _("Enter a custom command..."));
+
+    g_signal_connect (entry, "changed",
+                      G_CALLBACK (custom_entry_changed_cb),
+                      chooser);
+
+    chooser->details->custom_entry = entry;
+
+    button = gtk_file_chooser_button_new (_("Custom application"), GTK_FILE_CHOOSER_ACTION_OPEN);
+    g_signal_connect (button, "file-set",
+                      G_CALLBACK (custom_app_set_cb),
+                      chooser);
+    gtk_widget_show (button);
+    gtk_box_pack_start (GTK_BOX (custom_box), button, FALSE, FALSE, 6);
+
+    chooser->details->file_button = button;
+
+    gtk_widget_show_all (custom_box);
+
+    chooser->details->custom_picker = button;
 
 	box = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
 	gtk_box_set_spacing (GTK_BOX (box), 6);
-	gtk_button_box_set_layout (GTK_BUTTON_BOX (box), GTK_BUTTONBOX_END);
+	gtk_button_box_set_layout (GTK_BUTTON_BOX (box), GTK_BUTTONBOX_CENTER);
 	gtk_box_pack_start (GTK_BOX (chooser), box, FALSE, FALSE, 6);
 	gtk_widget_show (box);
 
-	button = gtk_button_new_with_label (_("Reset"));
-	g_signal_connect (button, "clicked", 
-			  G_CALLBACK (reset_clicked_cb),
-			  chooser);
-	gtk_widget_show (button);
-	gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
-	gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (box), button, TRUE);
+    GtkFileFilter *filter = gtk_file_filter_new ();
+    gtk_file_filter_add_custom (filter,
+                                GTK_FILE_FILTER_FILENAME,
+                                (GtkFileFilterFunc) exec_filter_func,
+                                NULL,
+                                NULL);
+    gtk_file_filter_set_name (filter, _("Executables"));
+    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (button), filter);
 
-	button = gtk_button_new_from_stock (GTK_STOCK_ADD);
+ 	button = gtk_button_new_with_label (_("Add to list"));
 	g_signal_connect (button, "clicked", 
 			  G_CALLBACK (add_clicked_cb),
 			  chooser);
@@ -401,6 +559,13 @@ nemo_mime_application_chooser_build_ui (NemoMimeApplicationChooser *chooser)
 
 	chooser->details->set_as_default_button = button;
 
+    button = gtk_button_new_with_label (_("Reset to system defaults"));
+    g_signal_connect (button, "clicked", 
+              G_CALLBACK (reset_clicked_cb),
+              chooser);
+    gtk_widget_show (button);
+    gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
+
 	/* initialize sensitivity */
 	info = gtk_app_chooser_get_app_info (GTK_APP_CHOOSER (chooser->details->open_with_widget));
 	if (info != NULL) {
@@ -418,6 +583,8 @@ nemo_mime_application_chooser_init (NemoMimeApplicationChooser *chooser)
 
 	gtk_orientable_set_orientation (GTK_ORIENTABLE (chooser),
 					GTK_ORIENTATION_VERTICAL);
+
+    chooser->details->custom_info = NULL;
 }
 
 static void
@@ -441,6 +608,8 @@ nemo_mime_application_chooser_finalize (GObject *object)
 
 	g_free (chooser->details->uri);
 	g_free (chooser->details->content_type);
+    if (chooser->details->custom_info != NULL)
+        g_object_unref (chooser->details->custom_info);
 
 	G_OBJECT_CLASS (nemo_mime_application_chooser_parent_class)->finalize (object);
 }
@@ -536,3 +705,18 @@ nemo_mime_application_chooser_new (const char *uri,
 
 	return chooser;
 }
+
+GAppInfo *
+nemo_mime_application_chooser_get_info (NemoMimeApplicationChooser *chooser)
+{
+    if (chooser->details->custom_info == NULL)
+        return gtk_app_chooser_get_app_info (GTK_APP_CHOOSER (chooser->details->open_with_widget));
+    else
+        return chooser->details->custom_info;
+}
+
+const gchar *
+nemo_mime_application_chooser_get_uri (NemoMimeApplicationChooser *chooser)
+{
+    return chooser->details->uri;
+}
