@@ -110,8 +110,60 @@ struct _NemoApplicationPriv {
 
 	gboolean no_desktop;
 	gchar *geometry;
+
+    NotifyNotification *unmount_notify;
 };
 
+void
+nemo_application_notify_unmount_done (NemoApplication *application,
+                                                  const gchar *message)
+{
+    if (application->priv->unmount_notify) {
+        notify_notification_close (application->priv->unmount_notify, NULL);
+        g_clear_object (&application->priv->unmount_notify);
+    }
+
+    if (message != NULL) {
+        NotifyNotification *unplug;
+        gchar **strings;
+
+        strings = g_strsplit (message, "\n", 0);
+        unplug = notify_notification_new (strings[0], strings[1],
+                                          "media-removable");
+
+        notify_notification_show (unplug, NULL);
+        g_object_unref (unplug);
+        g_strfreev (strings);
+    }
+}
+
+void
+nemo_application_notify_unmount_show (NemoApplication *application,
+                                          const gchar *message)
+{
+    gchar **strings;
+
+    strings = g_strsplit (message, "\n", 0);
+
+    if (!application->priv->unmount_notify) {
+        application->priv->unmount_notify =
+                        notify_notification_new (strings[0], strings[1],
+                                                 "media-removable");
+
+        notify_notification_set_hint (application->priv->unmount_notify,
+                                      "transient", g_variant_new_boolean (TRUE));
+        notify_notification_set_urgency (application->priv->unmount_notify,
+                                         NOTIFY_URGENCY_CRITICAL);
+    } else {
+        notify_notification_update (application->priv->unmount_notify,
+                                    strings[0], strings[1],
+                                    "media-removable");
+    }
+
+    notify_notification_show (application->priv->unmount_notify, NULL);
+    g_strfreev (strings);
+}
+
 static gboolean
 check_required_directories (NemoApplication *application)
 {
@@ -209,59 +261,6 @@ menu_provider_init_callback (void)
         nemo_module_extension_list_free (providers);
 }
 
-static void
-mark_desktop_files_trusted (void)
-{
-	char *do_once_file;
-	GFile *f, *c;
-	GFileEnumerator *e;
-	GFileInfo *info;
-	const char *name;
-	int fd;
-	
-	do_once_file = g_build_filename (g_get_user_data_dir (),
-					 ".converted-launchers", NULL);
-
-	if (g_file_test (do_once_file, G_FILE_TEST_EXISTS)) {
-		goto out;
-	}
-
-	f = nemo_get_desktop_location ();
-	e = g_file_enumerate_children (f,
-				       G_FILE_ATTRIBUTE_STANDARD_TYPE ","
-				       G_FILE_ATTRIBUTE_STANDARD_NAME ","
-				       G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE
-				       ,
-				       G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
-				       NULL, NULL);
-	if (e == NULL) {
-		goto out2;
-	}
-	
-	while ((info = g_file_enumerator_next_file (e, NULL, NULL)) != NULL) {
-		name = g_file_info_get_name (info);
-		
-		if (g_str_has_suffix (name, ".desktop") &&
-		    !g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE)) {
-			c = g_file_get_child (f, name);
-			nemo_file_mark_desktop_file_trusted (c,
-								 NULL, FALSE,
-								 NULL, NULL);
-			g_object_unref (c);
-		}
-		g_object_unref (info);
-	}
-	
-	g_object_unref (e);
- out2:
-	fd = g_creat (do_once_file, 0666);
-	close (fd);
-	
-	g_object_unref (f);
- out:	
-	g_free (do_once_file);
-}
-
 static void 
 selection_get_cb (GtkWidget          *widget,
 		  GtkSelectionData   *selection_data,
@@ -1146,6 +1145,66 @@ init_gtk_accels (void)
 			  G_CALLBACK (queue_accel_map_save_callback), NULL);
 }
 
+
+static void
+menu_state_changed_callback (NemoApplication *self)
+{
+    if (!g_settings_get_boolean (nemo_window_state, NEMO_WINDOW_STATE_START_WITH_MENU_BAR) &&
+        !g_settings_get_boolean (nemo_preferences, NEMO_PREFERENCES_DISABLE_MENU_WARNING)) {
+
+        GtkWidget *dialog;
+        GtkWidget *msg_area;
+        GtkWidget *checkbox;
+
+        dialog = gtk_message_dialog_new (NULL,
+                                         GTK_DIALOG_MODAL,
+                                         GTK_MESSAGE_INFO,
+                                         GTK_BUTTONS_OK,
+                                         _("Nemo's main menu is now hidden"),
+                                         NULL);
+
+        gchar *secondary;
+        secondary = g_strdup_printf (_("You have chosen to hide the main menu.  You can get it back temporarily by:\n\n"
+                                     "- Tapping the <Alt> key\n"
+                                     "- Right-clicking an empty region of the main toolbar\n"
+                                     "- Right-clicking an empty region of the status bar.\n\n"
+                                     "You can restore it permanently by selecting this option again from the View menu."));
+        g_object_set (dialog,
+                      "secondary-text", secondary,
+                      NULL);
+        g_free (secondary);
+
+        msg_area = gtk_message_dialog_get_message_area (GTK_MESSAGE_DIALOG (dialog));
+        checkbox = gtk_check_button_new_with_label (_("Don't show this message again."));
+        gtk_box_pack_start (GTK_BOX (msg_area), checkbox, TRUE, TRUE, 2);
+
+        g_settings_bind (nemo_preferences,
+                         NEMO_PREFERENCES_DISABLE_MENU_WARNING,
+                         checkbox,
+                         "active",
+                         G_SETTINGS_BIND_DEFAULT);
+
+        gtk_widget_show_all (dialog);
+
+        g_signal_connect (dialog, "response",
+                          G_CALLBACK (gtk_widget_destroy), NULL);
+    }
+
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 static void
 nemo_application_startup (GApplication *app)
 {
@@ -1202,6 +1261,9 @@ nemo_application_startup (GApplication *app)
 	g_signal_connect_object (self->priv->volume_monitor, "mount_added",
 				 G_CALLBACK (mount_added_callback), self, 0);
 
+    g_signal_connect_swapped (nemo_window_state, "changed::" NEMO_WINDOW_STATE_START_WITH_MENU_BAR,
+                              G_CALLBACK (menu_state_changed_callback), self);
+
 	/* Check the user's ~/.nemo directories and post warnings
 	 * if there are problems.
 	 */
@@ -1217,6 +1279,8 @@ nemo_application_quit_mainloop (GApplication *app)
 	nemo_icon_info_clear_caches ();
  	nemo_application_save_accel_map (NULL);
 
+    nemo_application_notify_unmount_done (NEMO_APPLICATION (app), NULL);
+
 	G_APPLICATION_CLASS (nemo_application_parent_class)->quit_mainloop (app);
 }
 
