@@ -254,8 +254,9 @@ create_bookmarks_window (NemoBookmarkList *list, GObject *undo_manager_source)
 	bookmarks = list;
 
 	builder = gtk_builder_new ();
+    gtk_builder_set_translation_domain (builder, GETTEXT_PACKAGE);
 	if (!gtk_builder_add_from_resource (builder,
-					    "/org/nemo/nemo-bookmarks-window.ui",
+					    "/org/nemo/nemo-bookmarks-window.glade",
 					    NULL)) {
 		return NULL;
 	}
@@ -797,8 +798,7 @@ update_bookmark_from_text (void)
 			(gtk_entry_get_text (GTK_ENTRY (uri_field)));
 		
 		bookmark = nemo_bookmark_new (location,
-						  name_text_changed ? gtk_entry_get_text (GTK_ENTRY (name_field)) : NULL,
-						  NULL);
+						  name_text_changed ? gtk_entry_get_text (GTK_ENTRY (name_field)) : NULL);
 		
 		g_object_unref (location);
 
